/*-----------------------------------------------------------------------------
 * Name:    ADC_MCB9F500.c
 * Purpose: A/D Converter interface for MCB9F500 evaluation board
 * Rev.:    1.00
 *----------------------------------------------------------------------------*/

/* Copyright (c) 2013 - 2014 ARM LIMITED

   All rights reserved.
   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions are met:
   - Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   - Redistributions in binary form must reproduce the above copyright
     notice, this list of conditions and the following disclaimer in the
     documentation and/or other materials provided with the distribution.
   - Neither the name of ARM nor the names of its contributors may be used
     to endorse or promote products derived from this software without
     specific prior written permission.
   *
   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
   ARE DISCLAIMED. IN NO EVENT SHALL COPYRIGHT HOLDERS AND CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
   POSSIBILITY OF SUCH DAMAGE.
   ---------------------------------------------------------------------------*/

#include "mb9b500r.h"                     // Device header
#include "Board_ADC.h"

#define ADC_RESOLUTION        12          /* Number of A/D converter bits     */
#define ADC_VALUE_MAX        (0xFFF)

static volatile uint16_t AD_last;         /* Last converted value             */
static volatile uint8_t  AD_done;         /* AD conversion done flag          */

/**
  \fn          int32_t ADC_Initialize (void)
  \brief       Initialize Analog-to-Digital Converter
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_Initialize (void) {

  /* configure Pins used for ADC                                              */
  FM3_GPIO->PFR1    &= ~(0x0001);         /* P10 is GPIO                      */
  FM3_GPIO->DDR1    &= ~(0x0001);         /* P10 is input                     */
  FM3_GPIO->ADE     |=  (0x0001);         /* P10 is analog input              */

  /* Setup and initialize ADC converter                                       */
  FM3_ADC0->ADCEN   |=  (0x01);           /* operation enable state           */ 
  while (!(FM3_ADC0->ADCEN & (1 << 1)));  /* wait until in operation state    */ 

  FM3_ADC0->ADCR     = ((1 << 3) |        /* enable Scan conversion interrupt */
                          (0 << 0) );
  FM3_ADC0->SFNS     =  (0x00);           /* only use one FIFO stage          */
  FM3_ADC0->SCIS0    =  (0x01);           /* chn. AN0 selected for conversion */
  FM3_ADC0->ADSS0   &= ~(0x01);           /* use ADST0 sampling time register */
  FM3_ADC0->ADST0    =  (0x09);           /* 2,5us @ 40MHz (25ns * (9+1) * 1) */

  NVIC_EnableIRQ(ADC0_IRQn);              /* enable ADC Interrupt             */
	return 0;
}


/**
  \fn          int32_t ADC_StartConversion (void)
  \brief       Start conversion
  \returns
   - \b  0: function succeeded
   - \b -1: function failed
*/
int32_t ADC_StartConversion (void) {

  FM3_ADC0->SCCR  =  (0x01);              /* Start A/D Conversion             */
	return 0;
}

/**
  \fn          int32_t ADC_ConversionDone (void)
  \brief       Check if conversion finished
  \returns
   - \b  0: conversion finished
   - \b -1: conversion in progress
*/
int32_t ADC_ConversionDone (void) {
  return (AD_done ? 0 : -1);
}


/**
  \fn          int32_t ADC_GetValue (void)
  \brief       Get converted value
  \returns
   - <b> >=0</b>: converted value
   - \b -1: conversion in progress or failed
*/
int32_t ADC_GetValue (void) {

  if (AD_done) {
    AD_done = 0;
    return AD_last;
  }
  return -1;
}

/**
  \fn          uint32_t ADC_GetResolution (void)
  \brief       Get resolution of Analog-to-Digital Converter
  \returns     Resolution (in bits)
*/
uint32_t ADC_GetResolution (void) {
  return ADC_RESOLUTION;
}

/**
  \fn          void ADC0_IRQHandler (void)
  \brief       Analog-to-Digital Converter Interrupt Handler
*/
void ADC0_IRQHandler(void) {

  FM3_ADC0->ADCR &= ~0x80;                /* clear SCIF bit                   */  
  AD_last = (FM3_ADC0->SCFD >> 20) & ADC_VALUE_MAX;
  AD_done = 1;

  FM3_ADC0->SCCR |= (0x10);               /* clear FIFO                       */
}
